@@ -12,11 +12,9 @@
 package ClamTk::App;
 
 use strict;
-#use warnings;    # disabled upon release
+# use warnings FATAL => 'all';
 $| = 1;
 
-use encoding 'utf8';
-
 use Net::DNS;
 use Date::Calc 'Date_to_Days';
 use File::Basename 'basename';
@@ -26,7 +24,7 @@ use Locale::gettext;
 
 sub get_TK_version {
     # Stick with %.2f format - 4.50 vice 4.5
-    return '4.35';
+    return '4.36';
 }
 
 sub get_path {
