@@ -21,13 +21,11 @@ package ClamTk::Device;
 # check what udev is reporting, and then see if it has a mountpoint.
 
 use strict;
-#use warnings;    # disabled upon release
+# use warnings FATAL => 'all';
 $| = 1;
 
 use Glib 'TRUE', 'FALSE';
 
-use encoding 'utf8';
-
 use Cwd 'realpath';
 use Locale::gettext;
 use POSIX 'locale_h';
