@@ -12,53 +12,56 @@
 package ClamTk::GUI;
 
 use strict;
-#use warnings;    # disabled upon release
+# use warnings FATAL => 'all';
 $| = 1;
 
-use encoding 'utf8';
-
 use Cwd 'abs_path';
+use File::Copy 'move';
 use File::Basename 'basename', 'dirname';
 use POSIX 'locale_h',          'strftime';
 use Date::Calc 'Delta_Days';
 use Digest::MD5 'md5_hex';
-use File::Find::Rule;
 use Locale::gettext;
-use File::Copy;
 use LWP::UserAgent;
+use File::Find::Rule;
+use Encode;
 
 use Gtk2;
 use Glib 'TRUE', 'FALSE';
 
-my $window;       # top level window
-my $top_label;    # status bar at the top
+my $window;       # Top level window
+my $top_label;    # Status bar at the top
 
-my ( @files, @quoted, @new );
-my %dirs_scanned;    # keeps track of directories scanned for display
-my $step = 0;        # step for progressbar
-my $pb;              # progressbar
-my $SCAN;            # scan handle; must be global for stop button
-my $scan_pid;        # pid of scan handle; must be global for stop button
-my $stopped = 0;     # program is stopped if 1, running if 0
-my $scan_frame;
-my $start_time;
+my $directive = '';    # Arguments passed to clamscan
+my %dirs_scanned;      # Keeps track of directories scanned for display
+my $pb;                # Progressbar
+my $step    = 0;       # ...with the progressbar ($pb)
+my $current = 0;       # ...with the progressbar ($pb)
+my $SCAN;              # Scan handle; must be global for stop button
+my $scan_pid;          # Pid of scan handle; must be global for stop button
+my $stopped = 1;       # Program is stopped if 1, running if 0
+my $found;             # Stores info about possibly infected files
+my $gui_tb;            # Easy buttons bar
+my $start_time;        # For use with Elapsed time
+my $scan_frame;        # Holds the progressbar, files scanned, etc.
+my $found_count = 0;   # Number of threats found
+my $num_scanned = 0;   # Number of files scanned, clean or bad
+my $av_remote   = '';  # ClamAV version, according to remote servers
+my $proxy_status_img;       # Image; global for 'flicker' appearance
+my ($last_infect_date);     # Date of last known threat found
+my ($scan_status_label);    # Shows what's being scanned, a little more
+
+# Scan frame stuff, under the progressbar
 my ( $left_status, $mid_status, $right_status );
-my $found;           # stores info about possibly infected files
-my ( $found_count, $num_scanned, $num_so_far, $current ) = (0) x 4;
 
+# Right-most column
 my ( $engine_version_text, $defs_version_text, $last_date_text );
-my ($last_infect_date);
+
+# Left-most column - global so we can adjust them throughout
 my ( $engine_gui_img, $defs_gui_img, $status_gui_img );
 
-my ($scan_status_label);
-my $av_remote = '';
-my $gui_tb;          # Easy buttons bar
-
-my ( @q_files,   @h_files );
 my ( $new_slist, $new_hlist );
 
-my $proxy_status_img;
-
 sub start_gui {
     INIT { Gtk2->init; }    ## no critic
 
@@ -91,43 +94,43 @@ sub start_gui {
 
         ### These fall under "Scan", although it's still called FileMenu. ###
         # Scan a single file
-        [   'Scan_File',        'gtk-copy',
+        [   'Scan_File',        undef,
             gettext('A _File'), '<control>F',
             undef, 	sub { ClamTk::GUI->getfile('file') },
 	    FALSE
         ],
         # Scan home directory, but do not descend
-        [   'Quick_Home', 		'gtk-go-down',
+        [   'Quick_Home', 		undef,
             gettext('Home (_Quick)'), 	'<control>Q',
             undef, 	sub { ClamTk::GUI->getfile('home') },
 	    FALSE
         ],
         # Scan home directory, descend (recursive)
-        [   'Full_Home', 		'gtk-goto-bottom',
+        [   'Full_Home', 		undef,
             gettext('Home (Recursive)'), 	'<control>Z',
             undef,	 sub { ClamTk::GUI->getfile('full-home') },
 	    FALSE
         ],
         # Scan a single directory
-        [   'Scan_Directory', 		'gtk-directory',
+        [   'Scan_Directory', 		undef,
             gettext('A _Directory'), 	'<control>D',
             undef,	sub { ClamTk::GUI->getfile('dir') },
 	    FALSE
         ],
         # Scan a directory, descend (recursive)
-        [   'Recursive_Scan', 		'gtk-sort-ascending',
+        [   'Recursive_Scan', 		undef,
             gettext('_Recursive Scan'), '<control>R',
             undef,	sub { ClamTk::GUI->getfile('recur') },
 	    FALSE
         ],
         # Scan a device - relies upon udev and /proc/mounts
-        [   'Scan_Device', 		'gtk-cdrom',
+        [   'Scan_Device', 		undef,
             gettext('A Device'), 	'<control>J', undef,	
 			sub { ClamTk::Device->look_for_device($window->get_position) },
 	    FALSE
         ],
         # Quit
-        [   'Exit',           'gtk-quit',
+        [   'Exit',           undef,
             gettext('E_xit'), '<control>X',
             undef,	sub { Gtk2->main_quit },
 	    FALSE
@@ -135,13 +138,13 @@ sub start_gui {
 
         ### These fall under View ###
         # Show dialog of recorded scans
-        [   'ManageHistories', 		'gtk-index',
+        [   'ManageHistories', 		undef,
             gettext('Manage _Histories'), "<control>H",
             undef, 	sub { history('delete') },
             FALSE
         ],
         # Closes the scan bar, zeroes out everything
-        [   'ClearOutput',                'gtk-clear',
+        [   'ClearOutput',                undef,
             gettext('Clear _Output'),     '<control>O',
             undef,	\&clear_output,
             FALSE
@@ -149,19 +152,19 @@ sub start_gui {
 
         ### These fall under Quarantine ###
         # Simple window showing number of quarantined objects
-        [   'Status', 			'gtk-edit', 
+        [   'Status', 			undef,
 	    gettext('_Status'), 	'<control>S',
             undef,	\&quarantine_check,
 	    FALSE
         ],
         # Dialog box - delete or return quarantined objects
-        [   'Maintenance', 		'gtk-preferences',
+        [   'Maintenance', 		undef,
             gettext('_Maintenance'), 	'<control>M',
             undef,	\&maintenance,
 	    FALSE
         ],
         # Delete quarantined objects
-        [   'Empty', 			'gtk-delete',
+        [   'Empty', 			undef,
             gettext('_Empty Quarantine Folder'), '<control>E',
             undef,	\&del_quarantined,
 	    FALSE
@@ -169,13 +172,13 @@ sub start_gui {
 
         ### These fall under Help ###
         # Dialog box to update signatures, check for GUI updates
-        [   'UpdateSig', 			'gtk-network',
+        [   'UpdateSig', 			undef,
             gettext('_Check for updates'), 	'<control>U',
 	    undef,	sub { ClamTk::Update->update_dialog($window->get_position) },
 	    FALSE
         ],
         # Standard About dialog
-        [   'About',                          'gtk-about',
+        [   'About',                          undef,
             gettext("_About"),                '<control>A',
             undef,	\&about,
 	    FALSE
@@ -183,26 +186,26 @@ sub start_gui {
 
         ### The final three here are for Advanced ###
         # Schedule scans and AV sig updates
-        [   'Scheduler', 		'gtk-media-forward',
+        [   'Scheduler', 		undef,
             gettext('_Scheduler'), 	'<control>T', undef,	
 		    sub { ClamTk::Schedule->schedule_dialog($window->get_position) },
 	    FALSE
         ],
         # Change to/from user updates to system
-        [   "AVsetup", 			'gtk-refresh',
+        [   "AVsetup", 			undef,
             gettext('Rerun antivirus setup _wizard'),
             '<control>W', undef,
             sub { ClamTk::Update->av_db_select($window->get_position) },
 	    FALSE
         ],
         # Huge preferences window
-        [   'Preferences',          'gtk-properties',
+        [   'Preferences',          undef,
             gettext('Preferences'), '<control>P',
             undef,	\&preferences,
 	    FALSE
         ],
 		# Submit a file for analysis
-        [   'SubmitFile',          'gtk-convert',
+        [   'SubmitFile',          undef,
             gettext('Submit a file for analysis'), '<control>Y',
 	    undef,	sub { ClamTk::Submit->analysis($window->get_position) },
 	    FALSE
@@ -280,7 +283,7 @@ sub start_gui {
     $gui_tb = Gtk2::Toolbar->new;
     $gui_tb->set_border_width(5);
     $gui_tb->set_style('both');
-	$gui_tb->set_show_arrow( FALSE );
+    $gui_tb->set_show_arrow(FALSE);
 
     # Button for quick home folder scan
     my $home_img = Gtk2::ToolButton->new_from_stock('gtk-home');
@@ -428,14 +431,9 @@ sub start_gui {
         [qw/shrink/], 0, 0 );
     $stop_btn->signal_connect(
         'clicked' => sub {
-
-            if ( !scalar(@files) ) {
-                $scan_frame->hide();
-            }
-            @quoted = ();
             kill 15, $scan_pid if ($scan_pid);
             $scan_status_label->set_text( gettext('Please wait...') );
-            waitpid( $scan_pid, 0 );
+            waitpid( $scan_pid, 0 ) if ($scan_pid);
 
             # this close returns the stupid readline() error.
             # not sure how to fix it yet, besides commenting
@@ -474,7 +472,7 @@ sub start_gui {
                     gettext('Could not scan (permissions)');
                 $found->{$found_count}->{action} = gettext('None');
                 $found_count++;
-                ClamTk::Results->display( 1, $found );
+                ClamTk::Results->display($found);
                 clean_up('cmd-scan');
             } elsif ( $ARGV[0] =~ m#^/(proc|sys|dev)# ) {
                 $found->{$found_count}->{name} = $ARGV[0];
@@ -482,7 +480,7 @@ sub start_gui {
                     gettext('Directory excluded from scan');
                 $found->{$found_count}->{action} = gettext('None');
                 $found_count++;
-                ClamTk::Results->display( 1, $found );
+                ClamTk::Results->display($found);
                 clean_up('cmd-scan');
             } else {
                 ClamTk::GUI->getfile( 'cmd-scan', $d );
@@ -755,19 +753,13 @@ sub getfile {
     my $paths = ClamTk::App->get_path('all');
     my %prefs = ClamTk::Prefs->get_all_prefs();
 
+    my @scans;    # file(s)/path(s) sent to clamscan
+    $directive = '';    # arguments sent to clamscan
+
     # Don't bother doing anything if clamscan can't be found
     warn "Cannot scan without clamscan!\n" unless $paths->{clampath};
     return unless $paths->{clampath};
 
-    # These lines are for 'thorough'. :)
-    # If it's selected, we add detection for both
-    # potentially unwanted applications and broken executables.
-    if ( $prefs{Thorough} ) {
-        $paths->{clamscan} .= ' --detect-pua --detect-broken';
-    } else {
-        $paths->{clamscan} =~ s/\s--detect-pua --detect-broken//;
-    }
-
     # Set the pulsebar to 0
     $pb->set_fraction(0);
     Gtk2->main_iteration while ( Gtk2->events_pending );
@@ -776,16 +768,20 @@ sub getfile {
 
     my ( $dir, $dialog );
     Gtk2->main_iteration while ( Gtk2->events_pending );
-    my $rule = File::Find::Rule->new;
 
-    # If we filter the results now, F:F:R should be much faster.
-    # We need to add the "not"'s first.
+    # We now only use this to count files,
+    # which makes the progressbar look good. I'd feel bad,
+    # but it doesn't seem to be any (much) slower...
+    my $rule = File::Find::Rule->new;
+    # For counting the files:
+    my @files;
 
     # By default, we ignore .gvfs directories.
     # Once we figure out KDE's process, we'll include that too.
     if ( !ClamTk::Prefs->get_preference('Mounted') ) {
         # This returned a '0', so filter the results
         for my $m (qw(.gvfs smb4k)) {
+            $directive .= " --exclude-dir=$m";
             $rule->not(
                 File::Find::Rule->name($m)->directory->prune->discard );
         }
@@ -797,26 +793,30 @@ sub getfile {
             ClamTk::Prefs->get_preference('Whitelist')
                 . $paths->{whitelist_dir} )
         ) {
-
-        # We just want the base directory name
+        # For clamscan, we'll stick with the full path:
+        my $full = $ignore;
+        # We just want the base directory name for F::F::R:
         $ignore = ( split( '/', $ignore ) )[-1];
+        $directive .= " --exclude-dir=$full";
         $rule->not(
-            File::Find::Rule->name($ignore)->directory->prune->discard );
+            File::Find::Rule->name("$ignore")->directory->prune->discard );
     }
 
     # Remove mail directories for now -
     # until we can parse them... sigh.
-    # http://clamtk.sourceforge.net/faq.html#inbox
+    # http://clamtk.sourceforge.net/faq.html#inbox .
+    # Not all of these can be appended to $HOME for a more
+    # specific path - kmail (e.g.) is somewhere under $HOME/.kde/blah/foo/...
     my @maildirs = qw(
         .thunderbird	.mozilla-thunderbird
         .evolution	Mail	kmail
         );
     for my $mailbox (@maildirs) {
+        $directive .= " --exclude-dir=$mailbox";
         $rule->not(
             File::Find::Rule->name($mailbox)->directory->prune->discard );
     }
 
-    # Now add what we're looking for.
     $rule->file;
     $rule->exists;
     $rule->readable;
@@ -825,6 +825,7 @@ sub getfile {
         or $option eq 'full-home'
         or $prefs{Recursive} );
 
+    # A more descriptive title based on what we're after
     my $title =
           ( $option eq 'file' ) ? gettext('Select File')
         : ( $option eq 'dir' && !$prefs{Recursive} )
@@ -833,10 +834,15 @@ sub getfile {
         ? gettext('Select a Directory (recursive scan)')
         : '';
 
+    # The ever-popular home directory scan
     if ( $option eq 'home' ) {
+        push( @scans, $paths->{directory} );
+        if ( !$prefs{Recursive} ) {
+            $directive .= ' --max-dir-recursion=1';
+        }
         $top_label->set_text( gettext('Please wait...') );
-        Gtk2->main_iteration while ( Gtk2->events_pending );
         @files = $rule->in( $paths->{directory} );
+        Gtk2->main_iteration while ( Gtk2->events_pending );
     } elsif ( $option eq 'file' ) {
         $dialog = Gtk2::FileChooserDialog->new(
             gettext($title), $window, 'open',
@@ -849,11 +855,10 @@ sub getfile {
             $top_label->set_text( gettext('Please wait...') );
             $window->queue_draw;
             Gtk2->main_iteration while ( Gtk2->events_pending );
+            push( @scans, $dialog->get_filenames );
             @files = $dialog->get_filenames;
-            $window->queue_draw;
             $dialog->destroy;
             $window->queue_draw;
-            Gtk2->main_iteration while ( Gtk2->events_pending );
         } else {
             $dialog->destroy;
             return;
@@ -863,10 +868,12 @@ sub getfile {
         return if ( not $dir or not -d $dir );
         Gtk2->main_iteration while ( Gtk2->events_pending );
         @files = $rule->in($dir);
-        Gtk2->main_iteration while ( Gtk2->events_pending );
+        push( @scans, $dir );
     } elsif ( $option eq 'full-home' ) {
         $top_label->set_text( gettext('Please wait...') );
         Gtk2->main_iteration while ( Gtk2->events_pending );
+        push( @scans, $paths->{directory} );
+        $directive .= ' --recursive';
         @files = $rule->in( $paths->{directory} );
     } elsif ( $option eq 'dir' or $option eq 'recur' ) {
         $dialog = Gtk2::FileChooserDialog->new(
@@ -888,7 +895,7 @@ sub getfile {
                     : gettext('Will not scan that directory');
                 $found->{$found_count}->{action} = gettext('None');
                 $found_count++;
-                ClamTk::Results->display( 1, $found );
+                ClamTk::Results->display($found);
                 clean_up('permissions');
                 return;
             }
@@ -900,6 +907,15 @@ sub getfile {
             $dir ||= $paths->{directory};
             Gtk2->main_iteration while ( Gtk2->events_pending );
             @files = $rule->in($dir);
+            push( @scans, $dir );
+
+            if ( !$prefs{Recursive} ) {
+                $directive .= ' --max-dir-recursion=1'
+                    if ( $option ne 'recur' );
+            }
+            if ( $option eq 'recur' ) {
+                $directive .= ' --recursive';
+            }
             Gtk2->main_iteration while ( Gtk2->events_pending );
         } else {
             $dialog->destroy;
@@ -911,10 +927,20 @@ sub getfile {
             $window->queue_draw;
             Gtk2->main_iteration while ( Gtk2->events_pending );
             @files = $rule->in( $cmd_input[0] );
+            push( @scans, $cmd_input[0] );
+            if ( !$prefs{Recursive} ) {
+                $directive .= ' --max-dir-recursion=1'
+                    if ( $option ne 'recur' );
+            }
         } else {
             $top_label->set_text( gettext('Please wait...') );
             $window->queue_draw;
             Gtk2->main_iteration while ( Gtk2->events_pending );
+            push( @scans, @cmd_input );
+            if ( !$prefs{Recursive} ) {
+                $directive .= ' --max-dir-recursion=1'
+                    if ( $option ne 'recur' );
+            }
             @files = @cmd_input;
         }
     } else {
@@ -932,17 +958,29 @@ sub getfile {
     $start_time = time if ( !$start_time );
     $right_status->set_text( gettext('Elapsed time: ') );
 
+    # we'll count this as ! $stopped
+    $stopped = 0;
+
     # reset %$found
     $found = {};
 
+    # These lines are for 'thorough'. :)
+    # If it's selected, we add detection for both
+    # potentially unwanted applications and broken executables.
+    if ( $prefs{Thorough} ) {
+        $directive .= ' --detect-pua --detect-broken';
+    } else {
+        $directive =~ s/\s--detect-pua --detect-broken//;
+    }
+
     # only a single file
     if ( $option eq 'file' ) {
-        scan( $option, @files );
+        scan( $directive, @scans );
     } else {
         # Pass the files to the filter first
-        @files = filter( $option, @files );
+        filter( $option, \@files );
 
-        # the pulse of the progressbar
+        # The pulse of the progressbar
         if ( scalar(@files) > 1 ) {
             $step = 1 / scalar(@files);
         } else {
@@ -951,88 +989,61 @@ sub getfile {
 
         # By default, 20Mb is the largest we go -
         # unless the preference is to ignore size.
-
         if ( !$prefs{SizeLimit} ) {
-            my @large;
-            foreach my $foo (@files) {
-                if ( -s $foo >= 20_000_000 ) {
-                    push( @large, $foo );
-                } else {
-                    push( @new, $foo );
-                }
-            }
-
-            # These files will be "skipped and assumed clean", which
-            # is exactly what the switch --max-filesize does.
-            if (@large) {
-                foreach my $too_big (@large) {
-                    #$scan_status_label->set_text(
-                    #sprintf gettext('Scanning %s...'),
-                    #dirname($too_big) );
-                    $num_scanned++;
-                    next;
-                }
-            }
-        } else {
-            @new = @files;
+            $directive .= ' --max-filesize=20M';
         }
+
         timer();
-
-        if (@new) {
-            # This seems ugly, but there are issues/limits giving
-            # arguments to commands. This still adds a pause after each
-            # 255 files sent, but works.
-            my @send;
-            while ( my $t = pop(@new) ) {
-                last if ($stopped);
-                push( @send, $t );
-                if ( scalar(@send) == 255 ) {
-                    scan( $option, @send );
-                    @send = ();
-                } else {
-                    next;
-                }
-            }
-            scan( $option, @send ) if (@send);
-        }
+        scan( $directive, @scans );
     }
-    clean_up($option);
+    clean_up();
 }
 
 sub filter {
-    my $opt    = shift;
-    my @filter = @_;
+    my ($opt)      = shift;
+    my ($file_ref) = @_;
 
     my $paths = ClamTk::App->get_path('all');
 
-    # remove the hidden files if chosen
+    # remove the hidden files if chosen:
     if (   !ClamTk::Prefs->get_preference('ScanHidden')
         && $opt ne 'recur'
         && $opt ne 'full-home' ) {
-        @filter = grep { basename($_) !~ /^\./ } @filter;
+        $directive .= ' --exclude="\/\."';
+        @$file_ref = grep { basename($_) !~ /^\./ } @$file_ref;
     }
 
-    # Attempt to remove dangling symlinks
-    my $index = 0;
-    while ( $index <= $#filter ) {
-        if ( -l $filter[$index] && !-e $filter[$index] ) {
-            splice( @filter, $index, 1 );
-        } else {
-            $index++;
+    # symlinks:
+    # The symlink stuff from clamscan requires >= 0.97.
+    my ($version) = ClamTk::App->get_AV_version();
+    # Ensure it's just digits and dots:
+    $version =~ s/[^0-9\.]//g;
+    if (   ( $version cmp '0.97' ) == 0
+        || ( $version cmp '0.97' ) == 1 ) {
+        $directive .= ' --follow-dir-symlinks=1';
+        $directive .= ' --follow-file-symlinks=1';
+        # Attempt to remove dangling symlinks
+        my $index = 0;
+        while ( $index <= $#{$file_ref} ) {
+            if ( -l $file_ref->[$index] && !-e $file_ref->[$index] ) {
+                splice( @$file_ref, $index, 1 );
+            } else {
+                $index++;
+            }
         }
     }
-
-    return @filter;
 }
 
 sub scan {
-    my ( $option, @get ) = @_;
+    # $directive = options passed to clamscan
+    # @get		 = list of files/paths sent to clamscan
+    my ( $directive, @get ) = @_;
+    @get = map { quotemeta($_) } @get;
+
+    $window->queue_draw;
 
     timer();
 
-    # We need to quotemeta the filenames
-    @quoted = map { quotemeta($_) } @get;
-
     my $paths   = ClamTk::App->get_path('all');
     my $command = $paths->{clamscan};
 
@@ -1043,21 +1054,22 @@ sub scan {
 
     # implicit fork; gives us the PID of clamscan so we can
     # kill it if the user hits the Stop button
-    $scan_pid = open( $SCAN, '-|', "$command @quoted 2>&1" );
+    $scan_pid = open( $SCAN, '-|', "$command $directive @get 2>&1" );
     defined($scan_pid) or die "couldn't fork: $!\n";
-    my $scan_count = 0;
-    $scan_status_label->set_text(
-        sprintf gettext('Scanning %s...'),
-        $option ne 'file'
-        ? dirname( $get[$scan_count] )
-        : $get[$scan_count] );
 
+    # Using ':encoding(utf8)' screws with BiDi text
+    # binmode( $SCAN, ':encoding(utf8)' );
+    # Adding the ":bytes" layer prevents CentOS
+    # from choking on the BiDi text.  Doesn't seem necessary
+    # for some distros.
+    binmode( $SCAN, ':utf8:bytes' );
+
+    # Turn on Tooltips
     my $tt = Gtk2::Tooltips->new();
 
     Gtk2->main_iteration while Gtk2->events_pending;
     while (<$SCAN>) {
         Gtk2->main_iteration while Gtk2->events_pending;
-
         if ( $av_remote eq 'undef' && /ClamAV engine is outdated/ ) {
             $engine_gui_img->set_from_stock( 'gtk-no', 'small-toolbar' );
             $engine_version_text->set_text( gettext('Outdated') );
@@ -1066,6 +1078,7 @@ sub scan {
             Gtk2->main_iteration while ( Gtk2->events_pending );
         }
         next if (/^LibClamAV/);
+        next if (/^\s*$/);
         $top_label->set_text('');
 
         my ( $file, $status );
@@ -1075,18 +1088,22 @@ sub scan {
         } elsif (/(.*?): (OK)$/) {
             $file   = $1;
             $status = $2;
-        }    #else {
+        }    # else {
              #	warn "something else: file = <$file>, stat = <$status>\n";
+             # }
              #}
 
-        chomp($file)   if ( defined $file );
-        chomp($status) if ( defined $status );
-
         # Ensure the file is still there (things get moved)
         # and that it got scanned
-        next unless ( -e $file && $status );
+        next unless ( $file && -e $file && $status );
         next if ( $status =~ /module failure/ );
 
+        chomp($file)   if ( defined $file );
+        chomp($status) if ( defined $status );
+        $scan_status_label->set_text( sprintf gettext('Scanning %s...'),
+            dirname($file) )
+            if ($file);
+
         # Lots of temporary things under /tmp/clamav;
         # we'll just ignore them.
         $dirs_scanned{ dirname($file) } = 1
@@ -1126,15 +1143,13 @@ sub scan {
             $found_count++;
         }
 
-        $num_so_far = keys %$found;
-
         # If we have possible threats, highlight it with bold.
-        if ( $num_so_far > 0 ) {
+        if ( $found_count > 0 ) {
             $mid_status->set_markup(
-                sprintf gettext('<b>Threats Found: %d</b>'), $num_so_far );
+                sprintf gettext('<b>Threats Found: %d</b>'), $found_count );
         } else {
             $mid_status->set_text( sprintf gettext('Threats Found: %d'),
-                $num_so_far );
+                $found_count );
         }
         $num_scanned++;
 
@@ -1149,49 +1164,33 @@ sub scan {
             $current * 100 );
         Gtk2->main_iteration while ( Gtk2->events_pending );
 
+        # Refresh timer stats
         timer();
-
-        # Get ready for the next file to be scanned.
-        # This is important for the next step.
-        $scan_count++;
-
-        # We do this to show the name of the file being
-        # scanned in "real time". Otherwise it shows up late.
-        if ( defined( $quoted[$scan_count] ) ) {
-            Gtk2->main_iteration while ( Gtk2->events_pending );
-            $scan_status_label->set_text(
-                sprintf gettext('Scanning %s...'),
-                $option ne 'file'
-                ? dirname( $get[$scan_count] )
-                : basename( $get[$scan_count] ) );
-            Gtk2->main_iteration while ( Gtk2->events_pending );
-        }
     }
-
     timer();
 
-    # No more files - close the filehandle and end the progressbar
-    if ( !@new ) {
-        close($SCAN);    # or warn "Unable to close scanner! $!\n";
-        $pb->set_text( gettext('Percent complete: 100') );
-    }
+    # Done scanning - close filehandle and return to
+    # getfile() and then to clean-up
+    close($SCAN);    # or warn "Unable to close scanner! $!\n";
+    $pb->set_text( gettext('Percent complete: 100') );
     Gtk2->main_iteration while ( Gtk2->events_pending );
 }
 
 sub clear_output {
     # Return if there are files being scanned
-    return if ( scalar(@files) > 0 );
+    return if ( !$stopped );
 
     # Reset the progressbar
     $pb->set_fraction(0);
-
-    # Refresh the main window
-    $window->queue_draw;
+    $pb->set_text('');
 
     # Clear the text
     $top_label->set_text('');
     $scan_status_label->set_text('');
 
+    # Refresh the main window
+    $window->queue_draw;
+
     # Reset the bottom labels
     $left_status->set_text( gettext('Files Scanned: ') );
     $mid_status->set_text( gettext('Threats Found: ') );
@@ -1219,20 +1218,19 @@ sub timer {
 }
 
 sub clean_up {
-    my $option = shift;
-
-    # Ensure the progressbar has hit 100%...
+    # Clear the progressbar
     $pb->set_fraction(1.0);
 
     # ...and remove the text
     $pb->set_text("");
+    $top_label->set_text('');
 
     my $db_total = ClamTk::App->get_num_sigs();
     my $REPORT;    # filehandle for histories log
     my ( $mon, $day, $year ) = split / /, strftime( '%b %d %Y', localtime );
 
     # Save date of scan
-    if ( $num_so_far > 0 ) {
+    if ( $found_count > 0 ) {
         $last_infect_date->set_text("$day $mon $year");
         ClamTk::Prefs->set_preference( 'LastInfection', "$day $mon $year" );
         $window->queue_draw;
@@ -1257,8 +1255,8 @@ sub clean_up {
             print $REPORT "$list\n";
         }
         printf $REPORT gettext("\nFound %d possible %s (%d %s scanned).\n\n"),
-            $num_so_far,
-            $num_so_far == 1 ? gettext('threat') : gettext('threats'),
+            $found_count,
+            $found_count == 1 ? gettext('threat') : gettext('threats'),
             $num_scanned,
             $num_scanned == 1 ? gettext('file') : gettext('files');
     } else {
@@ -1271,9 +1269,9 @@ sub clean_up {
         sprintf gettext('Scanning complete (%d signatures)'), $db_total );
     $left_status->set_text( sprintf gettext('Files Scanned: %d'),
         $num_scanned );
-    if ( $num_so_far != 0 ) {
-        $mid_status->set_text( sprintf gettext('Threats Found: %d'),
-            $num_so_far );
+    if ( $found_count != 0 ) {
+        $mid_status->set_markup( sprintf gettext('<b>Threats Found: %d</b>'),
+            $found_count );
     }
     $right_status->set_text( gettext('Ready') );
     $window->queue_draw;
@@ -1282,7 +1280,7 @@ sub clean_up {
     # the filename and its status - if we're saving a log.
     my $lsize = 20;
     my $rsize = 20;
-    if ( $num_so_far == 0 ) {
+    if ( $found_count == 0 ) {
         print $REPORT gettext("No threats found.\n");
     } else {
         # Now get the longest lengths of the column contents.
@@ -1313,29 +1311,19 @@ sub clean_up {
     my $lastdate = ClamTk::App->lastscan();
     $last_date_text->set_text($lastdate);
 
-    if ($num_so_far) {
-        ClamTk::Results->display( 0, $found );
-    } elsif ( !keys %$found && $option eq 'cmd-scan' ) {
-        for my $f (@files) {
-            $found->{$found_count}->{name}   = $f;
-            $found->{$found_count}->{status} = gettext('Nothing detected');
-            $found->{$found_count}->{action} = gettext('None');
-            $found_count++;
-        }
-        ClamTk::Results->display( 1, $found );
+    # If threats are found, show the Results window.
+    if ($found_count) {
+        ClamTk::Results->display($found);
     }
 
     # reset things
-    $num_so_far   = 0;
     $num_scanned  = 0;
     $found_count  = 0;
     %dirs_scanned = ();
-    @files        = ();
-    @new          = ();
-    @quoted       = ();
-    $current      = 0;
-    $stopped      = 0;
+    $stopped      = 1;
     $start_time   = '';
+    $directive    = '';
+    $current      = 0;
 }
 
 sub show_message_dialog {
@@ -1362,9 +1350,9 @@ sub maintenance {
     my $new_vbox = Gtk2::VBox->new( FALSE, 0 );
     $main_win->get_content_area()->add($new_vbox);
 
-    my $paths = ClamTk::App->get_path('viruses');
-    @q_files = glob "$paths/*";
-    my $s_win = Gtk2::ScrolledWindow->new;
+    my $paths   = ClamTk::App->get_path('viruses');
+    my @q_files = glob "$paths/*";
+    my $s_win   = Gtk2::ScrolledWindow->new;
     $s_win->set_shadow_type('none');
     $s_win->set_policy( 'automatic', 'automatic' );
     $new_vbox->pack_start( $s_win, TRUE, TRUE, 0 );
@@ -1381,9 +1369,7 @@ sub maintenance {
     $restore_btn->set_label( gettext('Restore') );
     $new_bar->insert( $restore_btn, -1 );
     $restore_btn->signal_connect(
-        clicked => \&restore,
-        'false_pos'
-        );
+        clicked => sub { restore( \@q_files ), 'false_pos' } );
 
     my $m_sep = Gtk2::SeparatorToolItem->new;
     $m_sep->set_draw(FALSE);
@@ -1398,7 +1384,8 @@ sub maintenance {
     my $del_pos_btn = Gtk2::ToolButton->new_from_stock('gtk-delete');
     $del_pos_btn->set_is_important(TRUE);
     $new_bar->insert( $del_pos_btn, -1 );
-    $del_pos_btn->signal_connect( clicked => \&main_del_pos, 'false_pos' );
+    $del_pos_btn->signal_connect(
+        clicked => sub { main_del_pos( \@q_files ), 'false_pos' } );
 
     for my $opt (@q_files) {
         push @{ $new_slist->{data} }, basename($opt);
@@ -1408,6 +1395,7 @@ sub maintenance {
 }
 
 sub restore {
+    my ($q_ref) = @_;
     my @sel = $new_slist->get_selected_indices;
     return if ( !@sel );
 
@@ -1416,13 +1404,11 @@ sub restore {
     my $deref     = $sel[0];
     my $data      = $new_slist->{data}[$deref];
     my $top_dir   = $paths->{viruses} .= "/";
-    my $full_path = $top_dir .= $data->[0];
+    my $tdata     = encode( 'utf8', $data->[0] );
+    my $full_path = $top_dir .= $tdata;
     return if ( not exists $data->[0] );
 
-    #warn "basename = >", $data->[0], "<\n";
-    #warn "top dir = >$top_dir<\n";
-    #warn "full_path = >$full_path<\n";
-
+    # Slurp in the file for md5-ing:
     my $ctx = do {
         local $/ = undef;
         open( my $F, '<', $full_path ) or do {
@@ -1438,13 +1424,14 @@ sub restore {
     # By default, the final destination will be $HOME
     my $final_destination = $paths->{directory} . '/';
 
+    # We should have a record of the quarantined file
+    # so we can return it - its path and permissions:
     my ( $path, $perm ) = ClamTk::Prefs->restore( $md5, 'exists' );
     if ( !$path ) {
         # We didn't receive a path for this file, so
         # by default it's going $HOME
-        $path = $final_destination . $data->[0];
+        $path = $final_destination . $tdata;
     }
-    #warn "path after check =>$path<\n";
 
     if ($path) {
         if ( -e $path ) {
@@ -1456,11 +1443,12 @@ sub restore {
         }
     } else {
         # We have nothing to go on, so it's going $HOME
-        $final_destination .= $data->[0];
+        #$final_destination .= $data->[0];
+        $final_destination .= $tdata;
     }
-    #warn "full_path = >$full_path<, dest = >$final_destination<\n";
 
-    system( 'mv', $full_path, $final_destination ) == 0 or do {
+    # Move it back
+    move( $full_path, $final_destination ) or do {
         show_message_dialog( $window, 'warning', 'ok',
             "Couldn't move file to $final_destination: $!\n" );
         return;
@@ -1471,26 +1459,29 @@ sub restore {
     $perm ||= 644;
     chmod oct($perm), $final_destination;
 
+    # If it's still in the original location, something blew up:
     if ( -e $full_path ) {
         show_message_dialog( $window, 'error', 'ok',
             gettext('Operation failed.') );
         return;
     }
 
-    # Update restore file: remove the file from it
+    # Update restore file: remove the file from it:
     ClamTk::Prefs->restore( $md5, 'remove' );
 
-    # Remove the file from view
+    # Remove the file from view:
     splice @{ $new_slist->{data} }, $deref, 1;
 
+    # Show a success message:
     my $msg = sprintf gettext('Restored as %s.'), $final_destination;
     show_message_dialog( $window, 'info', 'ok', $msg );
 
     # Get a fresh listing of quarantined files
-    @q_files = glob "$paths->{viruses}/*";
+    @$q_ref = glob "$paths->{viruses}/*";
 }
 
 sub main_del_pos {
+    my ($q_ref) = @_;
     my @sel = $new_slist->get_selected_indices;
     return if ( !@sel );
     my $deref = $sel[0];
@@ -1502,6 +1493,7 @@ sub main_del_pos {
     my $full_path = $top_dir .= $data->[0];
     return if ( !-e $full_path );
 
+    # Slurp in the file for md5-ing:
     my $ctx = do {
         local $/ = undef;
         open( my $F, '<', $full_path ) or do {
@@ -1532,7 +1524,7 @@ sub main_del_pos {
     splice @{ $new_slist->{data} }, $deref, 1;
 
     # Get a fresh listing of files
-    @q_files = glob "$paths/*";
+    @$q_ref = glob "$paths/*";
 }
 
 sub quarantine_check {
@@ -1622,8 +1614,8 @@ sub del_quarantined {
 }
 
 sub history {
-    my $paths = ClamTk::App->get_path('history');
-    @h_files = glob "$paths/*.log";
+    my $paths   = ClamTk::App->get_path('history');
+    my @h_files = glob "$paths/*.log";
     if ( scalar(@h_files) > 1 ) {
         @h_files = history_sort(@h_files);
     }
@@ -1696,7 +1688,8 @@ sub history {
     my $del_single = Gtk2::ToolButton->new_from_stock('gtk-delete');
     $del_single->set_is_important(TRUE);
     $new_bar->insert( $del_single, -1 );
-    $del_single->signal_connect( clicked => \&history_del_single );
+    $del_single->signal_connect(
+        clicked => sub { history_del_single( \@h_files ) } );
 
     for my $opt (@h_files) {
         $opt =~ s/\.log//;
@@ -1717,6 +1710,7 @@ sub history_sort {
 }
 
 sub history_del_single {
+    my ($h_ref) = @_;
     my @sel = $new_hlist->get_selected_indices;
     return if ( !@sel );
     my $deref = $sel[0];
@@ -1738,7 +1732,7 @@ sub history_del_single {
         return;
     }
     splice @{ $new_hlist->{data} }, $deref, 1;
-    @h_files = glob "$history/*";
+    @$h_ref = glob "$history/*";
 
     # Update the Last scan field
     my $lastdate = ClamTk::App->lastscan();
@@ -1764,7 +1758,7 @@ sub view_box {
         undef, [qw( modal destroy-with-parent )] );
     #$view_win->set_default_response('close');
     $view_win->signal_connect( response => sub { $view_win->destroy } );
-    $view_win->set_default_size( 600, 350 );
+    $view_win->set_default_size( 800, 350 );
     $view_win->move( $x, $y );
 
     my $textview = Gtk2::TextView->new;
@@ -1796,7 +1790,7 @@ sub view_box {
     my $scroll_win = Gtk2::ScrolledWindow->new;
     $scroll_win->set_border_width(5);
     $scroll_win->set_shadow_type('none');
-    $scroll_win->set_policy( 'never', 'automatic' );
+    $scroll_win->set_policy( 'automatic', 'automatic' );
 
     $view_win->vbox->pack_start( $scroll_win, TRUE, TRUE, 0 );
     $scroll_win->add($textview);
@@ -2201,6 +2195,7 @@ sub preferences {
     }
     my $path = ClamTk::App->get_path('db');
     $path .= '/local.conf';
+
     if ( -f $path ) {
         if ( open( my $FH, '<', $path ) ) {
             while (<$FH>) {
