@@ -12,17 +12,11 @@
 package ClamTk::Prefs;
 
 use strict;
-#use warnings;    # disabled upon release
+# use warnings FATAL => 'all';
 $| = 1;
 
-use encoding 'utf8';
-
-use ClamTk::App;
-use ClamTk::Submit;
-
 use Digest::MD5 'md5_hex';
 use File::Path 'mkpath';
-use File::Basename 'basename';
 use Date::Calc 'Date_to_Days';
 
 use Locale::gettext;
@@ -72,7 +66,7 @@ sub structure {
     }
 
     # This is /home/user/.clamtk/prefs,
-    # a custom INI-style file
+    # a custom INI-style file.
     if ( !-e $paths->{prefs} ) {
         warn "note: (re)creating prefs file.\n";
         open( my $F, '>:encoding(UTF-8)', $paths->{prefs} )
@@ -292,7 +286,7 @@ sub set_proxy {
 
     my $path = ClamTk::App->get_path('db');
 
-    # I'm going to clobber this every time.
+    # This gets clobbered every time.
     # Doesn't need to be utf-8 friendly. I think.
     open( my $FH, '>', "$path/local.conf" )
         or return -1;
@@ -321,6 +315,7 @@ sub restore {
         warn "Can't open restore file for reading: $!\n";
         return -1;
     };
+    binmode( $F, ':utf8' );
 
     while (<$F>) {
         chomp;
@@ -352,6 +347,7 @@ sub restore {
             warn "Can't open restore file for writing: $!\n";
             return -1;
         };
+        binmode( $F, ':utf8' );
 
         if ( scalar( keys %p ) ) {
             for my $e ( keys %p ) {
@@ -367,6 +363,7 @@ sub restore {
             warn "Can't open restore file for writing: $!\n";
             return -1;
         };
+        binmode( $F, ':utf8' );
 
         for my $e ( keys %p ) {
             next if ( $e eq $wanted );
