@@ -12,14 +12,13 @@
 package ClamTk::Results;
 
 use strict;
-#use warnings;    # disabled upon release
+# use warnings FATAL => 'all';
 $| = 1;
 
-use encoding 'utf8';
-
 use File::Basename 'basename', 'dirname';
-use File::Copy;
+use File::Copy 'move';
 use Digest::MD5 'md5_hex';
+use Encode;
 
 use Gtk2::SimpleList;
 use Glib 'TRUE', 'FALSE';
@@ -27,16 +26,19 @@ use Glib 'TRUE', 'FALSE';
 use Locale::gettext;
 use POSIX 'locale_h';
 
-my $popup;
 my $slist;
 my $hover_label;
+my $hash;
 
 sub display {
     shift;    # don't need package name
-    my $cmd_scan = shift;
-    my ($hash) = @_;
+    ($hash) = @_;
 
-    $popup = Gtk2::Dialog->new( gettext('Scanning Results'),
+    for my $k ( sort keys %$hash ) {
+        $hash->{$k}->{display} = encode( 'utf8', $hash->{$k}->{name} );
+    }
+
+    my $popup = Gtk2::Dialog->new( gettext('Scanning Results'),
         undef, 'destroy-with-parent' );
     $popup->signal_connect( destroy => sub { $popup->destroy } );
     $popup->set_default_size( 650, 250 );
@@ -48,12 +50,8 @@ sub display {
     my $pbox = Gtk2::VBox->new();
     $popup->get_content_area()->add($pbox);
     $hover_label =
-        ( $cmd_scan == 0 )
-        ? Gtk2::Label->new( gettext('Possible threats have been found.') )
-        : ( $cmd_scan == 1 )
-        ? Gtk2::Label->new( gettext('The scan is complete.') )
-        : '';
-    $pbox->pack_start( $hover_label, FALSE, FALSE, 0 );
+        Gtk2::Label->new( gettext('Possible threats have been found.') );
+    $pbox->pack_start( $hover_label, FALSE, TRUE, 0 );
 
     # This scrolled window holds the slist
     my $scrolled_win = Gtk2::ScrolledWindow->new;
@@ -73,7 +71,9 @@ sub display {
     $slist->set_rules_hint(TRUE);
     $slist->set_headers_clickable(TRUE);
 
-    $slist->set_reorderable(TRUE);
+    # No longer reorderable due to the way we
+    # push these onto the $slist->{data}.
+    # $slist->set_reorderable(TRUE);
     map { $_->set_resizable(TRUE) } $slist->get_columns;
     map { $_->set_sizing('fixed') } $slist->get_columns;
     map { $_->set_expand(TRUE) } $slist->get_columns;
@@ -91,20 +91,33 @@ sub display {
         );
 
     # Display the possible infections
-    for my $keys ( sort keys %$hash ) {
-        push @{ $slist->{data} },
-            [
-            $hash->{$keys}->{name}, $hash->{$keys}->{status},
-            $hash->{$keys}->{action},
+    #for my $keys ( sort keys %$hash ) {
+    my $i = 0;
+    while ( $i <= scalar keys %$hash ) {
+        push @{ $slist->{data} }, [
+            #$hash->{$keys}->{display}, $hash->{$keys}->{status},
+            #$hash->{$keys}->{action},
+            $hash->{$i}->{display}, $hash->{$i}->{status},
+            $hash->{$i}->{action},
             ];
+        $i++;
         Gtk2->main_iteration while ( Gtk2->events_pending );
+        last if ( $i == scalar keys %$hash );
     }
 
     my $tooltips = Gtk2::Tooltips->new;
     $tooltips->set_tip( $slist,
         gettext('Select a file and right-click for options...') );
 
-    # below: the right-click functionality. also uses 'sub confirm'.
+    # Left-click will clear the top label
+    $slist->signal_connect(
+        button_press_event => sub {
+            my ( $widget, $event ) = @_;
+            return FALSE unless $event->button == 1;
+            $hover_label->set_text('');
+        } );
+
+    # Right-click functionality. Also uses 'sub confirm'.
     $slist->signal_connect(
         button_press_event => sub {
             my ( $widget, $event ) = @_;
@@ -239,7 +252,8 @@ sub main_confirm {
     my $number  = shift;
     my $do_this = shift;
 
-    my $full_name = $slist->{data}[$number][0];
+    #my $full_name = $slist->{data}[$number][0];
+    my $full_name = $hash->{$number}->{name};
     my $status    = $slist->{data}[$number][1];
     my $action    = $slist->{data}[$number][2];
 
@@ -272,7 +286,7 @@ sub main_confirm {
         my $confirm_message = gettext('Really delete this file?');
         $confirm_message .= ' ' . '(' . basename($full_name) . ')';
         my $confirm =
-            Gtk2::MessageDialog->new( $popup, [qw(modal destroy-with-parent)],
+            Gtk2::MessageDialog->new( undef, [qw(modal destroy-with-parent)],
             'question', 'ok-cancel', $confirm_message );
 
         if ( 'cancel' eq $confirm->run ) {
@@ -305,29 +319,36 @@ sub main_confirm {
 }
 
 sub main_slist_delete {
-    my $number    = shift;
-    my $full_name = $slist->{data}[$number][0];
+    my $number = shift;
+    #my $full_name = $slist->{data}[$number][0];
+    my $full_name = $hash->{$number}->{display};
     my $status    = $slist->{data}[$number][1];
     my $action    = $slist->{data}[$number][2];
     $slist->{data}[$number][0] =
         "<span foreground='#CCCCCC'>$full_name</span>";
     $slist->{data}[$number][1] = "<span foreground='#CCCCCC'>$status</span>";
     $slist->{data}[$number][2] = "<span foreground='#CCCCCC'>$action</span>";
-    $popup->queue_draw;
+    #$popup->queue_draw;
     return;
 }
 
 sub move_to_quarantine {
+    # $number = row we're dealing with
     my $number = shift;
 
-    my $full_name = $slist->{data}[$number][0];
+    # We need the basename, because we're
+    # dealing with the filename, not the full_path
+    #my $full_name = $slist->{data}[$number][0];
+    #my $basename  = basename($slist->{data}[$number][0]);
+    my $full_name = $hash->{$number}->{name};
     my $basename  = basename($full_name);
 
+    # This is where threats go
     my $paths = ClamTk::App->get_path('viruses');
 
     # Ensure quarantine directory exists
     if ( not -e $paths or not -d $paths ) {
-        show_message_dialog( $popup, 'info', 'close',
+        show_message_dialog( undef, 'info', 'close',
             gettext('Quarantine directory does not exist.') );
         return;
     }
@@ -336,16 +357,16 @@ sub move_to_quarantine {
     my $mode = ( stat($full_name) )[2];
     my $perm = sprintf( "%03o", $mode & oct(7777) );
 
+    # Assign 600 permissions
     chmod oct(600), $full_name;
-    system( 'mv', $full_name, "$paths/$basename" ) == 0
-        or do {
+    move( $full_name, "$paths/$basename" ) or do {
         # When a 'mv' fails, it still probably did a 'cp'...
         # 'mv' copies the file first, then unlinks the source.
         # d'oh... so just to make sure, unlink the intended target
         # and THEN return.  No need to check for failure.
         unlink("$paths/$basename");
         return;
-        };
+    };
 
     # Update restore file by adding file, path and md5
     my $ctx = do {
