@@ -15,11 +15,9 @@ package ClamTk::Schedule;
 # scheduling, so we have call crontab as a system command.
 
 use strict;
-#use warnings;    # disabled upon release
+# use warnings FATAL => 'all';
 $| = 1;
 
-use encoding 'utf8';
-
 use POSIX 'locale_h';
 use Locale::gettext;
 
@@ -33,7 +31,6 @@ my $cmd =
     :                                   '';
 chomp($cmd);
 
-my $dialog;
 my ( $hour_s, $min_s, $hour_s_2, $min_s_2 );
 my ( $root_b, $home_b );
 my ( $scan_apply_btn, $scan_remove_btn );
@@ -265,8 +262,8 @@ sub is_enabled {
         $scan_apply_btn->set_sensitive(FALSE);
         $scan_remove_btn->set_sensitive(TRUE);
         $scan_status->set_text( gettext('A daily scan is scheduled.') );
-        if ($target) {
 
+        if ($target) {
             # We correctly (?) guessed the type of scan already set
             if ( $target eq 'system' ) {
 
@@ -325,12 +322,12 @@ sub apply_scan {
 
     my $paths = ClamTk::App->get_path('all');
 
-    # this probably isn't necessary;
+    # This probably isn't necessary;
     # ensure old task is removed
     remove('# clamtk-scan');
 
     my $tmp_file = "$paths->{clamtk}" . "/" . "cron";
-    open( my $T, '>', $tmp_file ) or do {
+    open( my $T, '>:encoding(UTF-8)', $tmp_file ) or do {
         win_dialog('Error opening temporary file');
         return;
     };
@@ -361,7 +358,7 @@ sub apply_scan {
             # So add the excludes
             $home .= ' --exclude-dir=' . $paths->{viruses};
             for ( split /;/, ClamTk::Prefs->get_preference('Whitelist') ) {
-                $home .= ' --exclude-dir=' . $_;
+                $home .= ' --exclude-dir=' . quotemeta($_);
             }
         }
     }
@@ -370,9 +367,8 @@ sub apply_scan {
         $root =
             '-r / --exclude-dir=/proc --exclude-dir=/sys --exclude-dir=/dev';
         if ( !$ignore_white->get_active ) {
-
-            # The box is not checked; use the whitelist
-            # So add the excludes
+            # The box is not checked; use the whitelist.
+            # So, add the excludes:
             $root .= ' --exclude-dir=' . $paths->{viruses};
             for ( split /;/, ClamTk::Prefs->get_preference('Whitelist') ) {
                 $root .= ' --exclude-dir=' . $_;
@@ -401,7 +397,7 @@ sub apply_scan {
     $full_cmd .= ' --log=$HOME/.clamtk/history/$(date +\%b-\%d-\%Y).log'
         . ' 2>/dev/null';
 
-    open( $T, '>>', $tmp_file ) or do {
+    open( $T, '>>:encoding(UTF-8)', $tmp_file ) or do {
         win_dialog('Error opening temporary file');
         return;
     };
@@ -429,7 +425,7 @@ sub apply_defs {
     remove('# clamtk-defs');
 
     my $tmp_file = $paths->{clamtk} . "/" . "cron";
-    open( my $T, '>', $tmp_file ) or do {
+    open( my $T, '>:encoding(UTF-8)', $tmp_file ) or do {
         win_dialog('Error opening temporary file');
         return;
     };
@@ -469,7 +465,7 @@ sub apply_defs {
         }
     }
 
-    open( $T, '>>', $tmp_file ) or do {
+    open( $T, '>>:encoding(UTF-8)', $tmp_file ) or do {
         win_dialog('Error opening temporary file');
         return;
     };
@@ -494,7 +490,7 @@ sub remove {
     my $paths = ClamTk::App->get_path('clamtk');
 
     my $tmp_file = "$paths/cron";
-    open( my $T, '>', $tmp_file ) or do {
+    open( my $T, '>:encoding(UTF-8)', $tmp_file ) or do {
         win_dialog('Error opening temporary file');
         return;
     };
@@ -536,7 +532,7 @@ sub win_dialog {
     $message .= $get;
     my $popup;
     $popup =
-        Gtk2::MessageDialog->new_with_markup( $dialog,
+        Gtk2::MessageDialog->new_with_markup( undef,
         [qw(modal destroy-with-parent)],
         'error', 'close', $message );
 
