@@ -10,11 +10,11 @@
 #
 # b) the "Artistic License".
 package ClamTk::Submit;
-# This file probably needs some work.  Or I need to understand locales
+# This file probably needs some work.  Or I need to understand encoding
 # better.  It's like one big hackjob.
 
 use strict;
-#use warnings;    #disabled upon release
+# use warnings FATAL => 'all';
 $| = 1;
 
 use LWP::UserAgent;
@@ -27,9 +27,13 @@ use Glib 'TRUE', 'FALSE';
 use Locale::gettext;
 use POSIX 'locale_h', 'strftime';
 
+# Highlights certain fields required by clamav.net's form:
 my $required_field = "<span foreground = 'red'>*</span>";
-my $chosen = '';    # file chosen by the user for analysis
 
+# file chosen by the user for analysis:
+my $chosen = '';
+
+# Main window:
 my $sub_window;
 my ( $name_entry,  $email_entry,  $desc_entry );
 my ( $name_desc,   $email_desc );
@@ -39,8 +43,7 @@ my ( $file_status, $long_string, $bottombar, $statusbar );
 
 # This file stores the number of times the
 # user has submitted a sample each day.
-my $file_path = ClamTk::App->get_path('submit');
-$file_path .= '/times';
+my ($file_path) = ClamTk::App->get_path('submit') . '/times';
 
 sub analysis {
     # These are the main $window's x/y coordinates.
