@@ -12,13 +12,12 @@
 package ClamTk::Update;
 
 use strict;
-#use warnings;    # disabled upon release
+# use warnings FATAL => 'all';
 $| = 1;
 
 use LWP::UserAgent;
 
-use encoding 'utf8';
-
+use File::Copy 'copy';
 use Locale::gettext;
 use POSIX 'locale_h';
 
@@ -590,17 +589,17 @@ sub save {
                 '/usr/local/share/clamav', '/var/db/clamav',
                 ) {
                 if ( -e "$dir_list/daily.cld" ) {
-                    system( 'cp', "$dir_list/daily.cld", "$paths/daily.cld" );
+                    copy( "$dir_list/daily.cld", "$paths/daily.cld" );
                     $d = 1;
                 } elsif ( -e "$dir_list/daily.cvd" ) {
-                    system( 'cp', "$dir_list/daily.cvd", "$paths/daily.cvd" );
+                    copy( "$dir_list/daily.cvd", "$paths/daily.cvd" );
                     $d = 1;
                 }
                 if ( -e "$dir_list/main.cld" ) {
-                    system( 'cp', "$dir_list/main.cld", "$paths/main.cld" );
+                    copy( "$dir_list/main.cld", "$paths/main.cld" );
                     $m = 1;
                 } elsif ( -e "$dir_list/main.cvd" ) {
-                    system( "cp", "$dir_list/main.cvd", "$paths/main.cvd" );
+                    copy( "$dir_list/main.cvd", "$paths/main.cvd" );
                     $m = 1;
                 }
                 last if ( $d && $m );
